import { ParamDefinition, ParamFamily } from "../param/param-definition";
import { ParamDomain, ParamDomainValue } from "../param/param-domain";
import { ParamsEquation } from "../param/params-equation";

/**
 * paramètres pour le calcul Lechapt et Calmon
 */
export class LechaptCalmonParams extends ParamsEquation {
    /** Débit */
    private _Q: ParamDefinition;

    /** Diamètre */
    private _D: ParamDefinition;

    /** Perte de charge */
    private _J: ParamDefinition;

    /** Longueur de la conduite */
    private _Lg: ParamDefinition;

    /** Paramètre de rugosité L */
    private _L: ParamDefinition;

    /** Paramètre de rugosité M */
    private _M: ParamDefinition;

    /** Paramètre de rugosité N */
    private _N: ParamDefinition;

    /** Perte de charge singulière Ks */
    private _Ks: ParamDefinition;

    constructor(rQ: number, rD: number, rJ: number, rLg: number, rL: number, rM: number, rN: number, rKs: number) {
        super();
        this._Q = new ParamDefinition(this, "Q", ParamDomainValue.POS, "m³/s", rQ, ParamFamily.FLOWS);
        this._D = new ParamDefinition(
            this, "D", new ParamDomain(ParamDomainValue.INTERVAL, 0, 20), "m", rD, ParamFamily.DIAMETERS
        );
        this._J = new ParamDefinition(this, "J", ParamDomainValue.POS_NULL, "m", rJ);
        this._Lg = new ParamDefinition(this, "Lg", ParamDomainValue.POS, "m", rLg, ParamFamily.LENGTHS);
        this._L = new ParamDefinition(this, "L", new ParamDomain(ParamDomainValue.INTERVAL, 0.8, 2), undefined, rL,
            undefined, false);
        this._M = new ParamDefinition(this, "M", new ParamDomain(ParamDomainValue.INTERVAL, 1.5, 2.5), undefined, rM,
            undefined, false);
        this._N = new ParamDefinition(this, "N", new ParamDomain(ParamDomainValue.INTERVAL, 4.5, 5.5), undefined, rN,
            undefined, false);
        this._Ks = new ParamDefinition(this, "Ks", ParamDomainValue.POS_NULL, undefined, rKs);

        this.addParamDefinition(this._Q);
        this.addParamDefinition(this._D);
        this.addParamDefinition(this._J);
        this.addParamDefinition(this._Lg);
        this.addParamDefinition(this._L);
        this.addParamDefinition(this._M);
        this.addParamDefinition(this._N);
        this.addParamDefinition(this._Ks);
    }

    get Q() {
        return this._Q;
    }

    get D() {
        return this._D;
    }

    get J() {
        return this._J;
    }

    get Lg() {
        return this._Lg;
    }

    get L() {
        return this._L;
    }

    get M() {
        return this._M;
    }

    get N() {
        return this._N;
    }

    get Ks() {
        return this._Ks;
    }
}
